from __future__ import print_function, division, absolute_import

import re

import astropy.units as u
from astropy.table import Table

import numpy as np

__all__ = ['parse_aij_table']


def parse_aij_table(table_name):
    """
    Return a list of objects, one for each source, from an AstroImageJ
    photometry table.

    Parameters
    ----------

    table_name : str
        Name of the table.
    """

    # Read in the raw table.
    if table_name.endswith('.csv'):
        # The table may have been edited and changed to csv.
        raw = Table.read(table_name)
    else:
        # The default, though, is tab-separated text with a file extension xls.
        raw = Table.read(table_name, format='ascii.tab')

    # Extract the names of all columns which are not specific to a source.
    # Source columns end with _TX or _CX where X is one or more digits.
    source_column = r'.*_[CT]\d+'
    common_columns = []
    for name in raw.colnames:
        if not re.search(source_column, name):
            if name.strip():
                common_columns.append(name)

    # Get all of the source designations from the names of the net counts
    # columns.
    flux_columns = [name for name in raw.colnames if
                    name.startswith('Source-Sky')]
    source_column_ids = [c.split('_')[1] for c in flux_columns]

    # For the first source, grab all of the column names that are specific to
    # a source. These will be the same for all sources.
    first_source_names = [name for name in raw.colnames if
                          name.endswith(source_column_ids[0])]
    generic_source_columns = [name.rsplit('_', 1)[0] for name in
                              first_source_names]

    # Make a list of star objects. Not sure this is actually better than
    # a list of tables or something simple like that.
    stars = []
    for idx, source in enumerate(source_column_ids):
        specifc_column_names = ['_'.join([g, source]) for g in
                                generic_source_columns]
        all_names = common_columns + specifc_column_names
        my_table = raw[all_names]
        for spec, gen in zip(specifc_column_names, generic_source_columns):
            my_table.rename_column(spec, gen)
        stars.append(Star(my_table, idx + 1))

    return stars


class Star(object):
    def __init__(self, table, id_num):
        self._table = table
        self._table['DEC'].unit = u.degree
        self.id = id_num

    @property
    def airmass(self):
        return self._table['AIRMASS']

    @property
    def counts(self):
        return self._table['Source-Sky']

    @property
    def ra(self):
        return self._table['RA'] / 24 * 360 * u.degree

    @property
    def dec(self):
        return self._table['DEC']

    @property
    def error(self):
        return self._table['Source_Error']

    @property
    def sky_per_pixel(self):
        return self._table['Sky/Pixel']

    @property
    def peak(self):
        return self._table['Peak']

    @property
    def jd_utc_start(self):
        return self._table['JD_UTC']

    @property
    def mjd_start(self):
        return self._table['J.D.-2400000'] - 0.5

    @property
    def exposure(self):
        try:
            return self._table['EXPOSURE']
        except KeyError:
            return self._table['EXPTIME']

    @property
    def magnitude(self):
        return -2.5 * np.log10(self.counts) + 2.5 * np.log10(self.exposure)

    @property
    def snr(self):
        return self.counts / self.error

    @property
    def magnitude_error(self):
        return 1.0857 / self.snr

    @property
    def bjd_tdb(self):
        return self._table['BJD_TDB']



from __future__ import print_function, division

import numpy as np

from ..magnitude_transforms import standard_magnitude_transform, filter_transform

import pytest

from astropy.table import Table, Column
from astropy.utils.data import get_pkg_data_filename


@pytest.mark.parametrize('bad_system', [None, 'monkeys'])
def test_filter_transform_bad_system(bad_system):
    fake_data = Table()
    with pytest.raises(ValueError) as e:
        filter_transform(fake_data, 'B', transform=bad_system)
    assert 'Must be one of' in str(e)
    assert str(bad_system) in str(e)


@pytest.mark.parametrize('system', ['ivezic', 'jester'])
def test_filter_transform(system):
    data_file = get_pkg_data_filename('data/mag_transform.csv')
    data = Table.read(data_file)
    in_system = data['system'] == system
    data = data[in_system]
    print(data)
    for output_filter in ['B', 'V', 'R', 'I']:
        f = filter_transform(data, output_filter, g='g', r='r', i='i',
                             transform=system)
        print(f)
        np.testing.assert_allclose(f, data[output_filter])


def test_no_matches():
    RAs = Column(name='RA',
                 data=[1, 2, 3],
                 unit='degree')
    i_Dec = Column(name='Dec',
                   data=[20, 20, 20],
                   unit='degree')

    c_Dec = Column(name='Dec',
                   data=[2, 2, 2],
                   unit='degree')

    R_mags = Column(name='R',
                    data=[15, 15, 15])
    e_R = Column(name='e_R',
                 data=[0.05, 0.1, 0.02])

    B_cat = Column(name='B',
                   data=[16, 16, 16])

    e_B_cat = Column(name='e_B',
                     data=[0.01, 0.04, 0.05])

    V_cat = Column(name='V',
                   data=[14, 15, 16])

    e_V_cat = Column(name='e_V',
                     data=[0.01, 0.04, 0.05])

    instrumental = Table([RAs, i_Dec, R_mags, e_R])
    catalog = Table([RAs, c_Dec, R_mags, B_cat, V_cat, e_R, e_B_cat, e_V_cat])

    # Test for fail if for_filter is omitted.
    with pytest.raises(ValueError) as e:
        standard_magnitude_transform(instrumental, catalog)
    assert 'Must provide a value for for_filter.' in str(e)

    # Test for appropriate error if filter is present in catalog but
    # not instrumental.
    with pytest.raises(ValueError) as e:
        standard_magnitude_transform(instrumental, catalog, for_filter='V')
    assert 'Filter V not found in instrumental table' in str(e)

    # Test that an error is raised when catalog has no matches to sources.
    with pytest.raises(ValueError) as e:
        standard_magnitude_transform(instrumental, catalog, for_filter='R')
    assert ('No matches found between instrumental and catalog '
            'tables.' in str(e))

    catalog['Dec'] = instrumental['Dec']
    # Add a 20 maagnitude offset...
    instrumental['R'] -= 20
    result = standard_magnitude_transform(instrumental, catalog, 'R')
    assert result
    print('=========>>>>>>', result['R'])
    np.testing.assert_allclose(result['R'][0], [0], atol=3e-5)
    np.testing.assert_allclose(result['R'][1], 20, rtol=1e-5)



# Licensed under a 3-clause BSD style license - see LICENSE.rst

"""
This is a package for doing stellar photometry that relies on astropy
"""

# Packages may add whatever they like to this file, but
# should keep this content at the top.
# ----------------------------------------------------------------------------
from ._astropy_init import *
# ----------------------------------------------------------------------------

if not _ASTROPY_SETUP_:
    from .core import *
    from .source_detection import *
    from .photometry import photutils_stellar_photometry
